//
//  NSURLCredential+RWPrivateKeyAdditions.h
//  RapidWeaver
//
//  Created by Keith Duncan on 22/05/2013.
//  Copyright (c) 2013 Realmac Software. All rights reserved.
//

#import <Foundation/Foundation.h>

@interface RMFTPURLCredential : NSObject

@property (copy, nonatomic) NSString *user;
@property (copy, nonatomic) NSString *password;
@property (readonly, nonatomic) BOOL hasPassword;

@property (copy, nonatomic) NSURL *privateKeyLocation;
@property (copy, nonatomic) NSURL *publicKeyLocation;

@property (copy, nonatomic) NSString *passphrase;
@property (copy, nonatomic) NSURL *localFolderURL;

@property (nonatomic) BOOL hasPrivateKey;
@property (nonatomic) BOOL isLocalFolderLocation;


+ (id)credentialWithUser:(NSString *)user password:(NSString *)password;

+ (id)credentialWithUser:(NSString *)user contentsOfPrivateKey:(NSURL *)privateKeyLocation publicKey:(NSURL *)publicKeyLocation passphrase:(NSString *)passphrase;
+ (id)credentialWithLocalFolderURL:(NSURL *)localFolderURL;


/*!
	\brief
	Atomically starts accessing all resources, if NO is returned none of the resources are available and `stopAccessingSecurityScopedResource` MUST NOT be called
 */
- (BOOL)startAccessingSecurityScopedResource:(NSError **)errorRef;

/*!
	\brief
	Must only be called if `startAccessingSecurityScopedResource:` returned YES
 */
- (void)stopAccessingSecurityScopedResource;

@end
